/* 
 * Copyright (c) 2014, Oracle and/or its affiliates. All Rights Reserved.
 */
"use strict";

(function() {
    var InventoryModule = angular.module("InventoryApp", []);
    
    InventoryModule.factory("familiesService", function($q, $http) {
        var _url = "rest/families";
        
        return {
            list: function() {
                var defered = $q.defer();
                $http.get(_url + "/list").success(function(families) {
                    defered.resolve(families);
                }).error(function(error) {
                    console.error("Error in fetching families..", error);
                    defered.reject();
                });
                return defered.promise;
            },
            
            add: function(family) {
                var defered = $q.defer();
                $http.put(_url + "/add", family).success(function() {
                    defered.resolve();
                }).error(function(error) {
                    console.error("Error in creating a new family: ", family, error);
                    defered.reject(error.error);
                });
                return defered.promise;
            }
        };
    });
    
    InventoryModule.factory("familyService", function($q, $http) {
        var _url = "rest/family/";
        return {
            getProducts: function(familyName) {
                var defered = $q.defer();
                $http.get(_url + familyName + "/products").success(function(products) {
                    defered.resolve(products);
                }).error(function(error) {
                    console.error("Error in getting products for family:", name, error);
                });
                return defered.promise;
            },
            
            get: function(name) {
                var defered = $q.defer();
                $http.get(_url + name).success(function(family) {
                    defered.resolve(family);
                }).error(function(error) {
                    console.error("Error in getting family:", name, error);
                });
                return defered.promise;
            }
        };
    });
    
    //Product Service for dealing with a product
    InventoryModule.factory("productService", function($q, $http) {
        var _url = "rest/product/";
        
        return {
            get: function(name) {
                var defered = $q.defer();
                $http.get(_url + name).success(function(p) {
                    defered.resolve(p);
                }).error(function(error) {
                    console.error(error);
                });
                return defered.promise;
            }
        };
    });
    
    // View Controller
    InventoryModule.controller("InventoryController", function($scope, $http, familiesService, familyService,
        productService) {
        $scope.view = "families";
        
        $scope.goTo = function(viewID) {
            $scope.view = viewID;
        };
        
        $scope.getFamilies = function() {
            familiesService.list().then(function(families) {
                $scope.families = families;
            });
        };
        $scope.getFamilies();
        
        $scope.onFamily = function(family) {
            $scope.currentFamily = family;
            familyService.getProducts(family.name).then(function(products) {
                $scope.products = products;
                $scope.goTo("products");
            });
        };

        $scope.onProduct = function(name) {
            productService.get(name).then(function(p) {
                $scope.currentProduct = p;
                $scope.goTo("product");
            });
        };

        $scope.onAddFamily = function() {
            $scope.newFamily = {};
            $scope.error = "";
            $scope.goTo("addFamily");
        };

        $scope.addFamily = function() {
            $scope.error = "";
            console.debug("Trying to add a family: ", $scope.newFamily);
            if(!$scope.newFamily.name) {
                $scope.error = "Provide a valid non-empty integer for the family name.";
                return false;
            } 
            if(isNaN(parseInt($scope.newFamily.name))) {
                $scope.error = "Family name has to be an integer.";
                return false;
            }
            $scope.error = "";
            $scope.newFamily.name = parseInt($scope.newFamily.name);
            familiesService.add($scope.newFamily).then(function() {
                $scope.getFamilies();
                $scope.goTo("families");
            }, function(error) {
                $scope.error = error;
            });
        };
    });
}());

