<?xml version="1.0" encoding="US-ASCII"?>
<!--

    DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

    Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.

    The contents of this file are subject to the terms of the GNU General
    Public License Version 2 only ("GPL"). You may not use this file except
    in compliance with the License.  You can obtain a copy of the License at
    https://avatar.java.net/license.html or legal/LICENSE.txt.  See the
    License for the specific language governing permissions and limitations
    under the License.

    When distributing the software, include this License Header Notice in each
    file and include the License file at legal/LICENSE.txt.

    GPL Classpath Exception:
    Oracle designates this particular file as subject to the "Classpath"
    exception as provided by Oracle in the GPL Version 2 section of the License
    file that accompanied this code.

    Modifications:
    If applicable, add the following below the License Header, with the fields
    enclosed by brackets [] replaced by your own identifying information:
    "Portions Copyright [year] [name of copyright owner]"

-->

<!--
  - Avatar HTML to XML transformation
  - @author spericas@java.net
  -
  - In what follows, prefix 'x' refers to the XHTML namespace and prefix 'a' to
  - the Avatar namespace.
  -
  - Tranformation rules: 
  -
  - (1) If x:body does not contain any x:div sections with data-widget='view'
  - attributes, a single a:view is created in the output; whether this view
  - becomes the main view in the application is controlled by the 'data-main'
  - attribute in x:html. The title of the output view is copied from the
  - x:title sub-element of x:head. The id of the output view is the base name
  - of the input HTML file.
  -
  - (2) An x:script tag that contains a 'data-model' or 'data-controller' attribute
  - is mapped to an Avatar model or controller type, respectively. An x:script tag
  - that contains a 'data-instance' and either a 'data-type' or a 'data-controller-type'
  - attribute is mapped to an Avatar model or controller instance, respectively.
  -
  - (3) x:input is mapped to a:htmlInput, a:htmlCheckbox or a:htmlRadio depending
  - on its type attribute and only if it contains a 'data-value' attribute.
  -
  - (4) x:select is mapped to a:htmlSelect if it defines a 'data-value' attribute.
  -
  - (5) Text nodes that are not in a mapped x:script elements are copied.
  -
  - (6) An HTML element that defines a 'data-widget' attribute is mapped to the
  - widget based on this attribute's value. The 'data-' prefix is removed from
  - each of the element's attributes. For example:
  -
  - <div data-widget="input" data-label="First Name" data-value="#{name.first}">
  - ==>> <a:input label="First Name" value="#{name.first}"/>
  -
  - It is also possible to specify property values in a single 'data-props'
  - attribute using a key/value pair syntax. This avoid repeating the 'data-'
  - prefix for every property. For example:
  -
  - <div data-widget="input" data-props="label: 'First Name', value: #{name.first}">
  - ==>> <a:input label="First Name" value="#{name.first}"/>
  -
  - (7) Any HTML element that does not match any of the rules above is copied
  - to the output without changes.
  -
  - Reference:
  -
  - o x:html   -> data-main, data-newview
  - o x:head   -> data-priority
  - o x:script -> (data-model | data-controller), data-namespace,
                  data-instance, (data-type | data-controller-type)
  - o x:div    -> data-widget, data-props, data-*
  - o x:input  -> data-value
  - o x:select -> data-value
  - o x:textarea -> data-value
  -
  -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
              xmlns:a="http://www.glassfish.org/avatar"
              xmlns:x="http://www.w3.org/1999/xhtml"
              xmlns:util="http://xml.apache.org/xalan/java/org.glassfish.avatar.util.Html2XmlUtil"
              exclude-result-prefixes="util">

    <!-- Output of transformation is XML -->
    <xsl:output method="xml"/>

    <!-- Name of HTML file excluding path but including extension -->
    <xsl:param name="fileName" select="'unnamed'"/>

    <!-- Default view name based on file name -->
    <xsl:variable name="viewName" select="util:patchName(substring-before($fileName, '.'))"/>
    
    <!-- Avatar namespace -->
    <xsl:variable name="avatarNs" select="'http://www.glassfish.org/avatar'"/>

    <!-- HTML title -->
    <xsl:variable name="title" select="/x:html/x:head/x:title"/>

    <!--
      - Main template matching element root. Outputs 'a:viewModule' and processes 
      - all model and controller type definitions first.
      -->
    <xsl:template match="/x:html">
        <!-- Define viewModule for this HTML file -->
        <a:viewModule>
            <!-- Process all attributes -->
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'data-main data-newview'"/>
            </xsl:call-template>

            <!-- All models and controller types should be at the beginning -->
            <xsl:call-template name="processModels"/>
            <xsl:call-template name="processControllers"/>

            <!-- Process the rest of the elements -->
            <xsl:apply-templates/>
        </a:viewModule>
    </xsl:template>

    <!--
      - Process all model definitions. Append "Model" to kind if necessary.
      -->
    <xsl:template name="processModels">
        <xsl:for-each select="//x:script[@data-model]">
            <xsl:variable name="name" select="util:patchModelKind(@data-model)"/>
            <xsl:call-template name="processModelsAndControllers">
                <xsl:with-param name="name" select="$name"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>

    <!--
      - Process all controller definitions.
      -->
    <xsl:template name="processControllers">
        <xsl:for-each select="//x:script[@data-controller]">
            <xsl:variable name="name" select="@data-controller"/>
            <xsl:call-template name="processModelsAndControllers">
                <xsl:with-param name="name" select="$name"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>

    <!--
      - Process model and controller definitions in script elements. If a new model type
      - is defined in an app, it can be referred to using the 'data-namespace' attribute,
      - so it must be supported here.
      -->
    <xsl:template name="processModelsAndControllers">
        <xsl:param name="name"/>

        <xsl:choose>
            <!-- If 'data-namespace' present, use that for element's namespace -->
            <xsl:when test="@data-namespace">
                <xsl:element name="{$name}" namespace="{@data-namespace}">
                    <xsl:call-template name="processAllAttrs">
                        <xsl:with-param name="skipAttrs" select="'data-model data-controller data-namespace'"/>
                    </xsl:call-template>
                    <xsl:value-of select="."/>
                </xsl:element>
            </xsl:when>
            <!-- Otherwise, assume Avatar's namespace -->
            <xsl:otherwise>
                <xsl:element name="{$name}" namespace="{$avatarNs}">
                    <xsl:call-template name="processAllAttrs">
                        <xsl:with-param name="skipAttrs" select="'data-model data-controller data-namespace'"/>
                    </xsl:call-template>
                    <xsl:value-of select="."/>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!--
      - Rule for x:head. Recursive processing changing the mode.
      -->
    <xsl:template match="x:head">
        <xsl:apply-templates mode="head"/>
    </xsl:template>

    <!--
      - Rule for x:link in x:head to load CSS stylesheets. Use data-priority in
      - x:head, if specified.
      -->
    <xsl:template match="x:link[@rel = 'stylesheet']" mode="head">
        <a:css>
            <!-- If x:head has a priority, set it here -->
            <xsl:if test="parent::x:head[@data-priority]">
                <xsl:attribute name="priority">
                    <xsl:value-of select="parent::x:head/@data-priority"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'rel type'"/>
            </xsl:call-template>
        </a:css>
    </xsl:template>

    <!--
      - Rule for x:script in body. Map it to a:script. Use data-priority in
      - x:head, if specified.
      -->
    <xsl:template match="x:script" mode="head">
            <xsl:call-template name="processScript"/>
    </xsl:template>

    <!--
      - Rule for x:title handle elsewhere.
      -->
    <xsl:template match="x:title" mode="head">
        <!-- no-op -->
    </xsl:template>

    <!--
      - Rule for x:body. Check if we have Avatar views or not. If we don't, create
      - a view using the base name of file as ID and the title specified in x:head.
      -->
    <xsl:template match="x:body">
        <xsl:variable name="views" select="x:div[@data-widget='view'] | a:view"/>
        <xsl:variable name="mainValue" select="/x:html/@data-main"/>
        <xsl:variable name="newPage" select="/x:html/@data-newview"/>

        <!-- If no Avatar views, generate single view -->
        <xsl:choose>
            <xsl:when test="string($newPage) != 'false' and count($views) = 0">
                <a:view id="{$viewName}" title="{$title}">
                    <xsl:if test="$mainValue">
                        <xsl:attribute name="main">
                            <xsl:value-of select="$mainValue"/>
                        </xsl:attribute>
                    </xsl:if>
                    <xsl:apply-templates mode="body"/>
                </a:view>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates mode="body"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!--
      - Rule for model instances. Simple mapping to a:model.
      -->
    <xsl:template match="x:script[@data-type]" mode="body">
        <a:model idref="{@data-type}">
            <!-- May not have @data-instance if defined in a:template -->
            <xsl:if test="@data-instance">
                <xsl:attribute name="id">
                    <xsl:value-of select="@data-instance"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'data-instance data-type'"/>
            </xsl:call-template>
        </a:model>
    </xsl:template>

    <!--
      - Rule for model instances. Simple mapping to a:controller.
      -->
    <xsl:template match="x:script[@data-controller-type]" mode="body">
        <a:controller idref="{@data-controller-type}">
            <!-- May not have @data-instance if defined in a:template -->
            <xsl:if test="@data-instance">
                <xsl:attribute name="id">
                    <xsl:value-of select="@data-instance"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'data-instance data-controller-type'"/>
            </xsl:call-template>
        </a:controller>
    </xsl:template>

    <!--
      - Model types handled in x:html.
      -->
    <xsl:template match="x:script[@data-model | @data-controller]" mode="body">
        <!-- no-op -->
    </xsl:template>

    <!--
      - Rule for x:script in body. Map it to a:script.
      -->
    <xsl:template match="x:script" mode="body">
        <xsl:call-template name="processScript"/>
    </xsl:template>

    <!--
      - Rule for x:input
      -->
    <xsl:template match="x:input[@data-value]" mode="body">
        <xsl:choose>
            <xsl:when test="@type = 'checkbox'">
                <a:htmlCheckbox>
                    <xsl:for-each select="@*">
                        <xsl:copy/>
                    </xsl:for-each>        
                </a:htmlCheckbox>                   
            </xsl:when>
            <xsl:when test="@type = 'radio'">
                <a:htmlRadio>
                    <xsl:for-each select="@*">
                        <xsl:copy/>
                    </xsl:for-each>
                </a:htmlRadio>
            </xsl:when>
            <xsl:otherwise>
                <a:htmlInput>
                    <xsl:for-each select="@*">
                        <xsl:copy/>
                    </xsl:for-each>
                </a:htmlInput>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!--
      - Rule for x:select elements. 
      -->
    <xsl:template match="x:select[@data-value]" mode="body">
        <a:htmlSelect>
            <xsl:for-each select="@*">
                <xsl:copy/>
            </xsl:for-each>
            <xsl:apply-templates mode="body"/>
        </a:htmlSelect>
    </xsl:template>

    <!--
      - Rule for x:textarea elements. Direct mapping to a:textarea.
      -->
    <xsl:template match="x:textarea[@data-value]" mode="body">
        <a:htmlTextarea>
            <xsl:for-each select="@*">
                <xsl:copy/>
            </xsl:for-each>
            <xsl:apply-templates mode="body"/>
        </a:htmlTextarea>
    </xsl:template>

    <!--
     - Rule for arbitrary x:div section with an @data-widget attribute naming
     - an avatar widget. The prefix 'data-' is dropped from each attribute.
     -->
    <xsl:template match="*[@data-widget]" mode="body">
        <xsl:variable name="elementNs">
            <xsl:choose>
                <xsl:when test="@data-namespace">
                    <xsl:value-of select="@data-namespace"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$avatarNs"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:element name="{@data-widget}" namespace="{$elementNs}">
            <!-- Process all attrs skipping those already processed -->
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'data-widget data-namespace'"/>
            </xsl:call-template>
            <xsl:apply-templates mode="body"/>
        </xsl:element>
    </xsl:template>

    <!--
      - Rule for all other elements not treated specially.
      -->
    <xsl:template match="*" mode="body">
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:choose>
                    <!-- Special case for all uiNode properties that are not HTML global attributes -->
                    <xsl:when test="name() = 'data-visible' or name() = 'data-textDir' or name() = 'data-controller'">
                        <xsl:attribute name="{substring-after(name(), 'data-')}">
                            <xsl:value-of select="."/>
                        </xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
            <xsl:apply-templates mode="body"/>
        </xsl:copy>
    </xsl:template>

    <!--
      - Ignore text outside x:head or x:body.
      -->
    <xsl:template match="text()"/>

    <!--
      - Text node in head are copied if not in <script data-model="..">.
      -->
    <xsl:template match="text()" mode="head">
        <xsl:if test="not(parent::*[@data-model])">
            <xsl:copy/>
        </xsl:if>
    </xsl:template>

    <!--
      - Text node in body are copied if not in <script data-model="..">.
      -->
    <xsl:template match="text()" mode="body">
        <xsl:if test="not(parent::*[@data-model])">
            <xsl:copy/>
        </xsl:if>
    </xsl:template>

    <!--
      - Process x:script elements.
      -->
    <xsl:template name="processScript">
        <a:script>
            <!-- If x:head has a priority, set it here -->
            <xsl:if test="parent::x:head[@data-priority]">
                <xsl:attribute name="priority">
                    <xsl:value-of select="parent::x:head/@data-priority"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'type'"/>
            </xsl:call-template>
            <xsl:value-of select="."/>
        </a:script>
    </xsl:template>

    <!--
      - Copies all attributes to the output.
      -->
    <xsl:template name="processAllAttrs">
        <xsl:param name="skipAttrs"/>

        <xsl:call-template name="processNonDataAttrs">
            <xsl:with-param name="skipAttrs" select="$skipAttrs"/>
        </xsl:call-template>
        <xsl:call-template name="processDataAttrs">
            <xsl:with-param name="skipAttrs" select="$skipAttrs"/>
        </xsl:call-template>
    </xsl:template>

    <!--
      - Copies all attributes whose names do not contain 'data-' and are not
      - in the list of attributes to skip (skipAttrs param). List of attributes
      - to skip is just a string containing attribute names.
      -->
    <xsl:template name="processNonDataAttrs">
        <xsl:param name="skipAttrs"/>

        <xsl:for-each select="@*">
            <xsl:if test="not(contains(name(), 'data-')) and not(util:stringContains($skipAttrs, name()))">
                <xsl:copy/>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>

    <!--
      - Copies all attributes whose names contain 'data-' and are not in the
      - list of attributes to skip (skipAttrs param). List of attributes
      - to skip is just a string containing attribute names. 
      -->
    <xsl:template name="processDataAttrs">
        <xsl:param name="skipAttrs"/>
        
        <xsl:for-each select="@*">
            <xsl:choose>
                <xsl:when test="name() = 'data-props'">
                    <!-- Parse JSON syntax in attribute -->
                    <xsl:variable name="attrs" select="util:parseProperties(.)"/>
                    <xsl:for-each select="$attrs">
                        <xsl:copy/>
                    </xsl:for-each>
                </xsl:when>
                <xsl:when test="contains(name(), 'data-') and not(util:stringContains($skipAttrs, name()))">
                    <xsl:attribute name="{substring-after(name(), 'data-')}">
                        <xsl:value-of select="."/>
                    </xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                    <!-- no-op -->
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>
    
</xsl:stylesheet>
